      subroutine bcast_parameters
C                                                       BCAST_PARAMETERS
C **********************************************************************
C *                                                               *
C * Subroutine: bcast_parameters                                  *
C * Author: Hong Lin (linh@cs.purdue.edu)                         *
C * Date:   Apr.20, 1998                                          *
C * Used in: aamain -- The main routine.                          *
C * Function: Broadcasting parameters read from input file or     *
C *           the file of pixel frames.                           *
C * Usage:                                                        *
C *   call bcast_parameters                                       *
C *                                                               *
C * Algorithm:                                                    *
C *   Firstly, define a structure type for MPI, rec_param,        *
C *   which is composed of a block of integers and a block of     *
C *   real numbers. This type will allow send/receive the         *
C *   parameters at one time.                                     *
C *                                                               *
C *   Then, Node 0 fills out the variable "rec_param_var" in type *
C *   "rec_param" with set parameters.                            *
C *                                                               *
C *   Node 0 broadcasts the parameters by calling "mpi_bcast".    *
C *                                                               *
C *   Lastly, nodes other than Node 0 set parameters with the     *
C *   values in record variable "rec_param_var".                  *
C *                                                               *
C * Modification history:                                         *
C *     - Hong Lin, Version 1.0, Aug.18, 1998                     *
C *     - Hong Lin, Version 2.0, Oct. 8, 1999                     *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *
C *                                                               *
C *****************************************************************
      implicit none
      include 'mpif.h'
      include '../include/allprogs.inc'
      include '../include/info.inc'
C-----------------------------------------------------------------------
      integer alloc_stat
      integer i,j ,mem1,mem2
C     
      integer*4, allocatable ::  tmp_int(:)
      real*4, allocatable    ::  tmp_real(:)

      call mpi_bcast (NFILES, 1, mpi_integer, 0,
     1                mpi_comm_world, mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, 'bcast_parameters',
     +  'Error in mpi_bcast ')

      mem1         = NFILES * 2 + 25
      mem2         = NFILES * 7 + 13

      allocate(tmp_int(mem1), tmp_real(mem2), stat = alloc_stat)

      if(alloc_stat .gt. 0) then
         call error_stop(progname, 'bcast_parameters',
     +        'Error in memory allocation')
      endif
       
      if (iam .eq. 0) then
        j  = 0
        tmp_int(j+1) = num_symm
        tmp_int(j+2) = min_x
        tmp_int(j+3) = max_x
        tmp_int(j+4) = min_y
        tmp_int(j+5) = max_y
        tmp_int(j+6) = min_z
        tmp_int(j+7) = max_z
        tmp_int(j+8) = max_views
        tmp_int(j+9) = npix_row
        tmp_int(j+10) = npix_col
        tmp_int(j+11) = num_den_pix
        tmp_int(j+12) = pixhd_endian
        tmp_int(j+13) = pixhd_htyp
        tmp_int(j+14) = pixhd_mode
        tmp_int(j+15) = CTFMODE
        tmp_int(j+16) = Filter
        tmp_int(j+17) = outmode
        tmp_int(j+18) = symm_code
        tmp_int(j+19) = view_perread 
        tmp_int(j+20) = NFILES 
        tmp_int(j+21) = BOX_OUT
        j  = 22
        do i = 1, NFILES 
          tmp_int(j) = NIDS(i) 
          j = j+1
        enddo
        do i = 1, NFILES 
          tmp_int(j) = NBOX(i) 
          j = j+1
        enddo
        tmp_int(j)     =  Nangle
        tmp_int(j+1)   =  funcmode
        tmp_int(j+2)   =  Ncenter
C
        j = 0
        tmp_real(j+1) = res_max
        tmp_real(j+2) = res_min
        tmp_real(j+3) = mapmagfactor
        tmp_real(j+4) = resolution
        tmp_real(j+5) = zero_fill
        tmp_real(j+6) = TEMPFAC
        tmp_real(j+7) = CTF_FF1
        tmp_real(j+8) = CTF_FF2
        j           = 9
        do i = 1, NFILES 
          tmp_real(j) = PIXSIZS(i) 
          j = j+1
        enddo
        do i = 1, NFILES
          tmp_real(j) = VOLTS(i)
          j = j+1
        enddo
        do i = 1, NFILES
          tmp_real(j) = DELF_MAJS(i)
          j = j+1
        enddo
        do i = 1, NFILES
          tmp_real(j) = DELF_MINS(i)
          j = j+1
        enddo
        do i = 1, NFILES
          tmp_real(j) = ANG_MAJS(i)
          j = j+1
        enddo
        do i = 1, NFILES
          tmp_real(j) = AMPFACS(i)
          j = j+1
        enddo
        do i = 1, NFILES
          tmp_real(j) = CS_COEFFS(i)
          j = j+1
        enddo
        tmp_real(j) = Dangle
        tmp_real(j+1) = funcweight
        tmp_real(j+2) = Dcenter
        tmp_real(j+3) = PIXSIZ_OUT

      endif
C
      call mpi_bcast (tmp_int, mem1, mpi_integer, 0,
     1                mpi_comm_world, mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, 'bcast_parameters',
     +  'Error in mpi_bcast ')

      call mpi_bcast (tmp_real, mem2, mpi_real, 0,
     1                mpi_comm_world, mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, 'bcast_parameters',
     +  'Error in mpi_bcast ')
C
      if (iam .gt. 0) then
        j  =  0
        num_symm  = tmp_int(j+1)
        min_x = tmp_int(j+2)
        max_x = tmp_int(j+3)
        min_y = tmp_int(j+4)
        max_y = tmp_int(j+5)
        min_z = tmp_int(j+6)
        max_z = tmp_int(j+7)
        max_views = tmp_int(j+8)
        npix_row  = tmp_int(j+9)
        npix_col  = tmp_int(j+10)
        num_den_pix  = tmp_int(j+11)
        pixhd_endian = tmp_int(j+12)
        pixhd_htyp   = tmp_int(j+13)
        pixhd_mode   = tmp_int(j+14)
        CTFMODE      = tmp_int(j+15)
        Filter       = tmp_int(j+16)
        outmode      = tmp_int(j+17)
        symm_code    = tmp_int(j+18)
        view_perread = tmp_int(j+19)
        NFILES       = tmp_int(j+20)
        BOX_OUT      = tmp_int(j+21) 
        j  = 22
        do i = 1, NFILES 
          NIDS(i)   = tmp_int(j)
          j = j+1
        enddo
        do i = 1, NFILES 
          NBOX(i)   = tmp_int(j)
          j = j+1
        enddo
        Nangle        = tmp_int(j)
        funcmode      = tmp_int(j+1)
        Ncenter       = tmp_int(j+2)
C
        j  = 0
        res_max     = tmp_real(j+1) 
        res_min     = tmp_real(j+2) 
        mapmagfactor= tmp_real(j+3)
        resolution  = tmp_real(j+4) 
        zero_fill   = tmp_real(j+5) 
        TEMPFAC     = tmp_real(j+6) 
        CTF_FF1     = tmp_real(j+7)
        CTF_FF2     = tmp_real(j+8)
        j           = 9
        do i = 1, NFILES
           PIXSIZS(i) = tmp_real(j)
          j = j+1
        enddo
        do i = 1, NFILES
          VOLTS(i)    = tmp_real(j) 
          j = j+1
        enddo
        do i = 1, NFILES
          DELF_MAJS(i)= tmp_real(j) 
          j = j+1
        enddo
        do i = 1, NFILES
          DELF_MINS(i) = tmp_real(j) 
          j = j+1
        enddo
        do i = 1, NFILES
          ANG_MAJS(i) = tmp_real(j)
          j = j+1
        enddo
        do i = 1, NFILES
          AMPFACS(i)  = tmp_real(j)
          j = j+1
        enddo
        do i = 1, NFILES
          CS_COEFFS(i)= tmp_real(j)
          j = j+1
        enddo
        Dangle          = tmp_real(j)
        funcweight      = tmp_real(j+1)
        Dcenter         = tmp_real(j+2)
        PIXSIZ_OUT      = tmp_real(j+3)

      endif

      deallocate(tmp_real, tmp_int)
      return
      end
