      subroutine exch_intpol(HKL_DFT, max_count, datalen,
     +     interpolants, hkl_min, hkl_max, max_iq_slab, mem_cmpt)
C
Cji *******************************************************************
Cji Subroutine exch_intpol used in cmpt_intrps
Cji Created by Yongchang Ji
Cji Function: exchange the data in HKL_DFT and store in interpolants
C * Input:                                                        *
C *   hkl_min (Integer) -- The minimum index of HKL space.        *
C *   hkl_max (Integer) -- The maximum index of HKL space.        *
C *   HKL_DFT (real)    -- the whole result of interpolation 
C *   max_count (integer)-- the maximum count of HKL_DFT in 1D
C *   datalen (integer) -- the end position of each HKL_DFT 
C * Output:           
C *   interpolants --- the result of interpolation of each slab
C *   
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *
C
Cji *******************************************************************
      implicit none

      include 'mpif.h'
      include '../include/allprogs.inc'

C     Subroutine arguments
      integer hkl_min, hkl_max, max_count, datalen, max_iq_slab
      real    HKL_DFT( 6,max_count)
      
#ifdef SPREC
      real  interpolants( 1:3, hkl_min:hkl_max,
     +hkl_min:hkl_max, frst_lst_iq(1,iam):frst_lst_iq(2,iam))
#else
      double precision  interpolants( 1:3, hkl_min:hkl_max,
     +hkl_min:hkl_max, frst_lst_iq(1,iam):frst_lst_iq(2,iam))
#endif
      
      double precision mem_cmpt

C     Local variables
C     segmlen( integer)--  used for data length which belongs to each slab
C     contlen( integer)-- used for each data length from all 
C     processes to owner process 

      integer i, j, ih, il, ik , mem_size, alloc_stat
      double precision mem_temp_DFT, mem_recvarr
      integer segmlen(0:num_nodes-1), contlen(0:num_nodes-1)
      integer positions(0:num_nodes-1),index(0:num_nodes)
      double precision   count,tmpctf
      
      real, allocatable :: temp_DFT(:), recvarr(:)
      
      mem_recvarr  = 0
      mem_temp_DFT = 0
      
      do i = 0, num_nodes-1
         segmlen(i) = 0
         positions(i)=1
         index(i)=1
      enddo
      
Cji count for data HKL_DFT belongs each slab and each start position
      do i = 1, datalen
         j = HKL_DFT(3,i) / max_iq_slab
         do while  ( HKL_DFT(3,i) .GT. frst_lst_iq(2,j))
            j = j + 1
         enddo
         segmlen(j) = segmlen(j) + 1
      enddo
      
Cji segment the data in HKL_DFT using temp_DFT according to slab.
      do i = 1, num_nodes-1
         positions(i)= positions(i-1) + segmlen(i-1)
         index(i)    = positions(i)
      enddo
      
      if (datalen .GT. 0) then
         mem_size        = (datalen +1) * 6
         allocate(temp_DFT(mem_size), stat = alloc_stat)
         mem_temp_DFT    = dble( 4 * mem_size)
         if(alloc_stat .gt. 0 ) call error_stop (progname,
     +        'exch_intpol', 'Error in allocating memory for temp_DFT')
         
         do i = 1, datalen
            j = HKL_DFT(3,i) / max_iq_slab
            do while  ( HKL_DFT(3,i) .GT. frst_lst_iq(2,j))
               j = j + 1
            enddo
            temp_DFT((index(j)-1)*6+1) =  HKL_DFT(1,i) 
            temp_DFT((index(j)-1)*6+2) =  HKL_DFT(2,i) 
            temp_DFT((index(j)-1)*6+3) =  HKL_DFT(3,i) 
            temp_DFT((index(j)-1)*6+4) =  HKL_DFT(4,i) 
            temp_DFT((index(j)-1)*6+5) =  HKL_DFT(5,i) 
            temp_DFT((index(j)-1)*6+6) =  HKL_DFT(6,i) 
            index(j) = index(j) + 1
         enddo
         do i = 1, datalen
            HKL_DFT(1,i) = temp_DFT((i-1)*6+1)
            HKL_DFT(2,i) = temp_DFT((i-1)*6+2)
            HKL_DFT(3,i) = temp_DFT((i-1)*6+3)
            HKL_DFT(4,i) = temp_DFT((i-1)*6+4)
            HKL_DFT(5,i) = temp_DFT((i-1)*6+5)
            HKL_DFT(6,i) = temp_DFT((i-1)*6+6)
         enddo
         deallocate(temp_DFT)
      endif
      
Cji exchange segmlen to contlen 
      call mpi_alltoall (segmlen, 1, mpi_integer, contlen,
     +     1, mpi_integer, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop(progname,'exch_intpol',
     +     'MPI_ALLTOALL error of segmlen')
      
Cji multicast the segmented data to each process, so that they have all
Cji the data belongs to its slab

      index(0) = 0
      do i = 1, num_nodes
         index(i) = index(i-1) + contlen(i-1)*6
         contlen(i-1) = contlen(i-1)*6
      enddo
      
      mem_size        = index(num_nodes)+6
      allocate(recvarr(mem_size), stat = alloc_stat)
      mem_recvarr     = dble( 4 * mem_size)
      if(alloc_stat .gt. 0 ) call error_stop ('cmpt_intrps',
     +     'exch_intpol', 'Error in allocating memory for recvarr')
      mem_cmpt = mem_cmpt + max(mem_temp_DFT,mem_recvarr)
      
      do i = 0, num_nodes-1
         call mpi_gatherv(HKL_DFT(1,positions(i)),segmlen(i)*6,
     +        mpi_real, recvarr, contlen, index,
     +        mpi_real, i, mpi_comm_world, mpi_err)
         if( mpi_err .ne. 0 ) call error_stop(progname,'exch_intpol',
     +        'MPI_GATHERV error ')
      enddo

Cji  fill the interpolatants slab
      do j = 0, index(num_nodes)/6 - 1
         ih = recvarr(j*6+1) 
         il = recvarr(j*6+2) 
         ik = recvarr(j*6+3) 
         
         if (interpolants(1,ih,il,ik).gt.0.0) then ! Already data at this pt
            count  = interpolants(1,ih,il,ik)
            tmpctf = recvarr(j*6+6)
            interpolants(2,ih,il,ik) = (count*interpolants(2,ih,il,ik)+
     +           recvarr(j*6+4) * tmpctf) / (count + tmpctf)
            interpolants(3,ih,il,ik) = (count*interpolants(3,ih,il,ik)+
     +           recvarr(j*6+5) * tmpctf) / (count + tmpctf)
            interpolants(1,ih,il,ik) = count + tmpctf
         else ! No data yet at this pt, interpolants(1,ih,il,ik) = 1.0
            interpolants(1,ih,il,ik) = recvarr(j*6+6)
            interpolants(2,ih,il,ik) = recvarr(j*6+4)
            interpolants(3,ih,il,ik) = recvarr(j*6+5)
         endif
      enddo
   
      deallocate(recvarr)
      return
      end subroutine exch_intpol
