      program p3dcut

      implicit none

      include 'mpif.h'
      include '../include/allprogs.inc'
      include '../include/info.inc'
      include '../include/com_01.inc'
      include '../include/mem_time.inc'
      
      character(len=4) :: subname = '    '

c     local variables, used only in the main routine
      real, allocatable :: map3d(:)
      
      integer in_rad, out_rad, cut_step, max_slab, mem_size,
     +     alloc_stat, ierr
      real    cut_weight, x1, x2, x3, x4

c     *********************************************************************
c     *                      initialization                               *
c     *********************************************************************
      test_time   = .true.
      test_memory = .true.
      call usage
      progname = 'pcut '

      call mpi_init ( mpi_err )
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'error in mpi_init')
      call mpi_comm_rank ( mpi_comm_world, iam, mpi_err )
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'error in mpi_comm_rank')
      call mpi_comm_size ( mpi_comm_world, num_nodes, mpi_err )
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'error in mpi_comm_size')
      if (num_nodes .gt. max_numnodes) then
         write(stdoutput, 9001) num_nodes, max_numnodes
 9001    format(//' requested number of nodes = num_nodes    = ',i10/
     +        ' available number of nodes = max_numnodes = ',i10/)
         call error_stop(progname, subname,
     +        'num_nodes requested exceeds max_numnodes')
      endif

      t1 = wtime()
      if ( iam.eq. 0) then
         print *, ' number of nodes = ', num_nodes

         read (*,'(a)') in_3d_name
         write(*,'(2a)') 'input map filename: ', in_3d_name

         call get_map_dimensions(in_3d_name, npix_col, npix_row, 
     $        angstrom_per_pix, ierr)
         if(ierr .gt. 0) then
            call error_stop('PCUT', 'PCUT',
     +           'Error in 3D map mode, it should be 20 or 40')
         endif

         read (*,'(a)') out_file_name
         write(*,'(2a)') 'output map filename: ', out_file_name

C     Read in parameters as reals and then implicitly convert to
C     integers. This make the input format a little more flexible since
C     we avoid potential runtime errors if integer values are specified
C     with a decimal point (e.g. 50.0 instead of 50)

         read (*,*) x1, x2, x3, x4
         in_rad = x1
         out_rad = x2
         cut_step = x3
         cut_weight = x4
         
c --------------------------------------------------------------------
         print *, 'inner radius, outer radius, cut step, drop weight:',
     +        in_rad, out_rad, cut_step, cut_weight
      endif
c --------------------------------------------------------------------

      if( num_nodes .gt. 1 ) then
         min_x     = in_rad
         max_x     = out_rad
         nangle    = cut_step
         dangle    = cut_weight
c     call bcast_parameters
c     *******************************
c     broadcast user's parameters and pif file header to other nodes.
         call bcast_parameters 

         in_rad     = min_x
         out_rad    = max_x
         cut_step   = nangle
         cut_weight = dangle
      endif

c     ---------- end of initializationl --- synchronize all the nodes ------
c     
      call mpi_barrier(mpi_comm_world, mpi_err)
      t2 = wtime()
      t_init = t2 - t1
      
c     **********************************************************************
c     *             3d dft of input 3d density map                         *
c     **********************************************************************
c
      call set_indices( num_nodes, 1, npix_row, frst_lst_y, max_slab)

      mem_size = npix_col * npix_row * max_slab
      allocate(map3d(mem_size), stat = alloc_stat)
      mem_map3d = dble(4 * mem_size) 
      if (alloc_stat .gt. 0) call error_stop (progname, subname,
     +     '3d density map memory allocation failed')
      mem_total = mem_total + mem_map3d
      mem_peak  = max(mem_peak, mem_total)

      call cut_map(map3d,max_slab,in_rad,out_rad,cut_step,cut_weight)

      deallocate(map3d)
      mem_total = mem_total - mem_map3d
      call mpi_barrier(mpi_comm_world, mpi_err)
      t1 = wtime()
      t_interp = t1 - t2

c ***************************************************************************

      if (mem_total .ne. 0) print *,'mem_total=',mem_total,
     +     'at the end, mem_total should be 0!!!'

      if ( iam .eq. 0 ) then
c     print timing message.

         time = t_init + t_interp

         if (test_time)
     +        print 40, iam, t_init,
     +        t_interp,
     +        time
 40      format (/ 'node ', i3,
     +        ' time used in initialization                      =',
     +        f15.4, ' ' / 8x,
     +        ' time used in read map and convert and write map  =',
     +        f15.4, ' ' / 8x,
     +        ' total time used                                  =',
     +        f15.4)

c     print memory allocating message.
         if (test_memory) print 60, iam, mem_peak, mem_map3d
 60      format(/ 'node ', i3, ' memory allocation in bytes:', /
     +        8x, ' peak memory allocated   =', f18.1 /
     +        8x, ' memory for 3d map       =', f18.1 )

         print *, '**********finished nicely**********'
      endif
      call mpi_finalize ( mpi_err )

      stop
      end program p3dcut
