c  ******************************************************************
c  *                                                                *
c  *           *****     subroutine pftcc_peak     *****            *
c  *                                                                *
c  *    subroutine find peak of cross-correlation "map".            *
c  *    modified 17-sep-2004 by mbs, put surface fitting            *
c  *    last modified 25-oct-2005 by mbs, bug fixed in xcen, ycen   *
c  ******************************************************************
c
      subroutine pftcc_peak (cc_map, ncol, nrow, xcen, ycen, fft_origx,
     $     fft_origy, flag, ipx, ipy, peakx1, peakx2, peaky1, peaky2,
     $     step1, step2)

      implicit none

      integer*4 icor,jcor,icor2,jcor2                                 !mbs
      parameter (icor=15,jcor=15)
      integer*4 ix, iy, ipx, ipy, ncol, nrow
      integer*4 i, j
      real*4 peakx1, peakx2, peaky1, peaky2, step1, step2

      integer*4 incx,incy, excx, excy, lpeakx,lpeaky,flag
      real*4    amps(icor,jcor), xcen, ycen, peak, fft_origx, fft_origy
      real*4    halfx, halfy, cc_map(ncol,nrow), peakx, peaky,lpeak

      real*4    map_out(ncol,nrow) !RSS

      integer*4 iopt/0/,mx,my,k, alloc_stat
      integer*4 kx/3/,ky/3/
      integer*4 nxest,nyest,nx,ny,lwrk,kwrk,ier
      integer*4 indx
      integer*4, allocatable :: iwrk(:)
      real*4    fit_max
      real*4    xb,xe,yb,ye,s,fp
      real*4    step, rangex, rangey
      real*4    xcoord(icor), ycoord(jcor)
      real*4, allocatable :: surf_fit(:)
      real*4, allocatable :: tx(:),ty(:),c(:),wrk(:)
      logical   init, first/.true./

c     --------------------------------------------------------------------

      incx  =  0.3 * ((ncol+1)/2)
      incy  =  0.3 * ((nrow+1)/2)
      excx  =  0.1 * ((ncol+1)/2)
      excy  =  0.1 * ((nrow+1)/2)

      icor2 = (icor + 1)/2                                     !mbs
      jcor2 = (jcor + 1)/2                                     !mbs      

c$$$      call map_peak (cc_map, ncol, nrow, ipx, ipy, peak)
c$$$      lpeak = peak

      call rearrange_fft_style(ncol, nrow, cc_map, map_out)
      cc_map = map_out !f90 array syntax

      call map_peak_restr (cc_map, ncol, nrow, ipx, ipy, peak,
     1           lpeakx,lpeaky,lpeak, incx,incy, excx, excy)

      flag = 0 ! need to initialize flag - not done anywhere else
      if(peak .ne. lpeak) then
         flag = -99
      endif
      halfx = float(ncol)/2.0
      halfy = float(nrow)/2.0

      do j = 1,jcor
        iy = ipy + (j-jcor2)
        if(iy.le.0)    iy = iy + nrow
        if(iy.gt.nrow) iy = iy - nrow
         ycoord(j) = iy
        do i = 1,icor
          ix = ipx + (i-icor2)
          if(ix.le.0)    ix = ix + ncol
          if(ix.gt.ncol) ix = ix - ncol
          if (j.eq.1) xcoord(i) = ix
          amps(j,i) = cc_map(ix,iy)   !mbs 10/26/05
        enddo
      enddo

      mx = icor
      my = jcor

      xb = xcoord(1)
      xe = xcoord(mx)
      yb = ycoord(1)
      ye = ycoord(my)

      s = 0.00001
      nxest = mx+kx+1
      nyest = my+ky+1

      step = 1.

      lwrk = 4+nxest*(my+2*kx+5)+nyest*(2*ky+5)+mx*(kx+1)+my*(ky+1)
     1     +max(my,nxest)
      kwrk = 3+mx+my+nxest+nyest

      allocate(surf_fit(mx*my), tx(nxest), ty(nyest),
     $     c((nxest-kx-1)*(nyest-ky-1)), wrk(lwrk),
     $     iwrk(kwrk), stat = alloc_stat)

      if (alloc_stat .gt. 0) then
         write(*,'(a)')'could not allocate memory in pftcc_peak'
         stop
      end if

      ier = 2
      do while(ier.ne.0) 
         call regrid(iopt,mx,xcoord,my,ycoord,amps,xb,xe,yb,ye,kx,ky,s,
     1      nxest,nyest,nx,tx,ny,ty,c,fp,wrk,lwrk,iwrk,kwrk,ier)
c$$$         write(*,*)'nx,ny,fp:',nx,ny,fp
c$$$         write(*,*)'mx,my:',mx,my
c$$$         write(*,*)'xcoord:',(xcoord(i),i = 1,mx)
c$$$         write(*,*)'ycoord:',(ycoord(i),i = 1,my)
c$$$         write(*,*)'tx:',(tx(i),i = 1,nx)
c$$$         write(*,*)'ty:',(ty(i),i = 1,ny)
c$$$         write(*,*)'spline coefficients:'
c$$$         do j = 1,ny-ky-1
c$$$            write(*,'(10f5.2)')(c((ny-ky-1)*(i-1)+j),i=1,nx-kx-1)
c$$$         end do
c$$$         write(*,*)"ier =", ier, "s =", s
         if(ier.eq.10) then
            write(*,*)"check dimensions etc. in regrid call"
            stop
         end if
         if(ier.eq.-2) then
            if(s.gt.1.e-6) then
             write(*,*)"smoothing factor too big, dividing by 10"
             s = s/10.
            else
             ier = 0
            end if
         else if(ier.eq.-1) then
c$$$            write(*,*)"interpolating spline returned"
            ier = 0
         else if(ier.eq.1) then
            write(*,*)"nxest or nyest are too small, increasing by 5"
            nxest = nxest + 5
            nyest = nyest + 5
         else if(ier.eq.2 .or. ier.eq.3) then
            write(*,*)"smoothing factor too small, increasing 2x"
            s = s*2.
         end if
      end do

      k = 0
      do while(k.lt.2)
         call bispev(tx,nx,ty,ny,c,kx,ky,xcoord,mx,ycoord,my,surf_fit,
     1      wrk,lwrk,iwrk,kwrk,ier)
         if(ier.eq.10) then
            write(*,*)"check dimensions etc. in bispev call"
            stop
         end if

         if(k.eq.0) then
            if(first) then
             init = .true.
             first = .not. first
            end if
         end if

c       initialize parameters
         fit_max = -1.e38
         peakx         = -10000.
         peaky         = -10000.
         do j = 1, my
            do i = 1, mx
             if(surf_fit(j+(i-1)*mx) .gt. fit_max) then
                fit_max = surf_fit(j+(i-1)*mx)
                peakx = xcoord(i)
                peaky = ycoord(j)
             end if
            end do
         end do

         if(k.eq.0) then
            peakx1 = peakx
            peaky1 = peaky
            step1 = step
         else
            peakx2 = peakx
            peaky2 = peaky
            step2 = step
         endif

         if(peakx .ne. -10000. .and. peaky .ne. -10000.) then
            step = step/10.
            rangex = step*(icor2-1)
            rangey = step*(jcor2-1)
            i = 1

c     the following block of code uses a scary real do-loop variable
c     rewriting to use integer variable
c            do xcen = peakx-rangex,peakx+rangex,step
c               xcoord(i) = xcen
c               i = i + 1
c            end do
c            i = 1
c            do ycen = peaky-rangey,peaky+rangey,step
c               ycoord(i) = ycen
c               i = i + 1
c            end do
            
            xcen = peakx - rangex
            do indx = 1, 2*icor2 - 1
               xcoord(i) = xcen
               xcen = xcen + step
               i = i + 1
            enddo
            i = 1
            ycen = peaky - rangey
            do indx = 1, 2*jcor2 - 1
               ycoord(i) = ycen
               ycen = ycen + step
               i = i + 1
            enddo
c     end rewritten block of code !rss

         else
            write(*,*) 'surface fit failed, max not found'
            stop
         end if
         k = k + 1
      end do

      if(init) init = .false.
      xcen = fft_origx - (peakx - halfx - 1) !mbs 10/25/05
      ycen = fft_origy - (peaky - halfy - 1) !mbs 10/25/05

      deallocate(surf_fit, tx, ty, c, wrk, iwrk)

      return
      end subroutine pftcc_peak
